@@ -24,6 +24,7 @@ from testflows._core.cli.arg.handlers.report.official import Handler as official
 from testflows._core.cli.arg.handlers.report.compare.handler import Handler as compare_handler
 from testflows._core.cli.arg.handlers.report.requirements import Handler as requirements_handler
 from testflows._core.cli.arg.handlers.report.map.handler import Handler as map_handler
+from testflows._core.cli.arg.handlers.report.metrics.handler import Handler as metrics_handler
 
 class Handler(HandlerBase):
     @classmethod
@@ -41,6 +42,7 @@ class Handler(HandlerBase):
         passing_handler.add_command(report_commands)
         fails_handler.add_command(report_commands)
         version_handler.add_command(report_commands)
+        metrics_handler.add_command(report_commands)
         requirements_handler.add_command(report_commands)
         srs_coverage_handler.add_command(report_commands)
         map_handler.add_command(report_commands)
