@@ -29,10 +29,10 @@ from collections import namedtuple
 import testflows.settings as settings
 
 from .exceptions import DummyTestException, ResultException, TestIteration, DescriptionError
-from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED
+from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED, MANDATORY
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
-from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, Attribute, Requirement
+from .objects import get, Null, OK, Fail, Skip, Error, PassResults, Argument, Attribute, Requirement, ArgumentParser
 from .objects import RepeatTest, ExamplesTable
 from .constants import name_sep, id_sep
 from .io import TestIO, LogWriter
@@ -213,6 +213,8 @@ class TestBase(object):
             self.flags = Flags(flags)
         self.cflags = Flags(cflags) | (self.flags & CFLAGS)
         self.uid = get(uid, self.uid)
+        if self.uid is not None:
+            self.uid = str(self.uid)
         self.xfails = get(xfails, None)
         self.xflags = get(xflags, None)
         self.only = get(only, None)
@@ -261,7 +263,7 @@ class TestBase(object):
 
     @classmethod
     def make_tags(cls, tags):
-        return set(get(tags, cls.tags))
+        return {str(tag) for tag in set(get(tags, cls.tags))}
 
     def _enter(self):
         self.io = TestIO(self)
@@ -300,8 +302,6 @@ class TestBase(object):
                 exception(exc_type, exc_value, exc_traceback, test=self)
                 result = Error("unexpected %s: %s" % (exc_type.__name__, str(exc_value)), test=self.name)
                 self.result = self.result(result)
-                if isinstance(exc_value, KeyboardInterrupt):
-                    raise result
 
         try:
             if exc_value is not None:
@@ -443,7 +443,7 @@ class TestDefinition(object):
     @classmethod
     def __create__(cls,  **kwargs):
         self = super(TestDefinition, cls).__new__(cls)
-        self.name = kwargs.pop("name", None)
+        self.name = None
         self.test = None
         self.parent = None
         self.kwargs = kwargs
@@ -459,9 +459,13 @@ class TestDefinition(object):
         :argparser: test specific argument parser
         :return: argument parser
         """
+        description = self.kwargs.get("description")
+        if description is not None:
+            description = str(description)
+
         main_parser = ArgumentParserClass(
             prog=sys.argv[0],
-            description=self.kwargs.get("description"),
+            description=description,
             description_prog="Test - Framework",
             epilog = epilog
         )
@@ -677,7 +681,7 @@ class TestDefinition(object):
             format_description = kwargs.pop("format_description", True)
 
             if not top():
-                cli_args = self._parse_cli_args(self._argparser(argparser))
+                cli_args = self._parse_cli_args(self._argparser(argparser if not isinstance(argparser, ArgumentParser) else argparser.value))
                 kwargs["args"].update({k: v for k,v in cli_args.items() if not k.startswith("_")})
 
             test = kwargs.pop("test", None)
@@ -688,7 +692,7 @@ class TestDefinition(object):
             if not issubclass(test, TestBase):
                 raise TypeError(f"{test} must be subclass of TestBase")
 
-            name = test.make_name(self.name, parent.name if parent else None, kwargs["args"], format=format_name)
+            name = test.make_name(kwargs.pop("name", None), parent.name if parent else None, kwargs["args"], format=format_name)
 
             if parent:
                 kwargs["parent"] = parent.name
@@ -717,10 +721,13 @@ class TestDefinition(object):
                 with parent.lock:
                     parent.child_count += 1
 
+            self.name = name
             self.tags = test.make_tags(kwargs.pop("tags", None))
             self.description = test.make_description(kwargs.pop("description", None), kwargs["args"], format=format_description)
             self.parent = parent
 
+            kwargs["flags"] = Flags(kwargs.get("flags"))
+
             # anchor all patterns
             kwargs["xfails"] = {
                 absname(k, name if name else name_sep): v for k, v in dict(kwargs.get("xfails") or {}).items()
@@ -737,10 +744,6 @@ class TestDefinition(object):
             kwargs["repeat"] = [RepeatTest(*r) for r in kwargs.get("repeat", [])] or None
             if kwargs["repeat"]:
                 [r.pattern.at(name if name else name_sep) for r in kwargs["repeat"]]
-            # should not skip Background, Given and Finally steps
-            if kwargs.get("subtype") in (TestSubType.Background, TestSubType.Given, TestSubType.Finally):
-                kwargs["only"] = kwargs["only"] or []
-                kwargs["only"].append(The(join(name, "*")))
 
             self._apply_xflags(name, kwargs)
             self._apply_start(name, parent, kwargs)
@@ -750,9 +753,28 @@ class TestDefinition(object):
             self._apply_end(name, parent, kwargs)
             self._apply_only(name, kwargs)
 
+            # for And subtype we change the subtype to be that of its sibling
+            if kwargs.get("subtype") is TestSubType.And:
+                sibling = previous()
+                if not sibling:
+                    raise TypeError("`And` subtype can't be used here as it has no sibling from which to inherit the subtype")
+                if sibling.type != kwargs["type"]:
+                    raise TypeError("`And` subtype can't be used here as it sibling is not of the same type")
+                kwargs["subtype"] = sibling.subtype
+
+            # should not skip Background, Given and Finally steps
+            if kwargs.get("subtype") in (TestSubType.Background, TestSubType.Given, TestSubType.Finally) or kwargs["flags"] & MANDATORY:
+                kwargs["flags"] &= ~SKIP
+                kwargs["only"] = None
+                kwargs["skip"] = None
+                kwargs["start"] = None
+                kwargs["end"] = None
+                kwargs["only_tags"] = None
+                kwargs["skip_tags"] = None
+
             if not top():
                 # can't skip, pause before or after top level test
-                kwargs["flags"] = Flags(kwargs.get("flags")) & ~SKIP
+                kwargs["flags"] &= ~SKIP
                 kwargs["flags"] &= ~PAUSE_BEFORE
                 kwargs["flags"] &= ~PAUSE_AFTER
 
@@ -821,9 +843,9 @@ class TestDefinition(object):
             return
 
         if not start.match(name):
-            kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
+            kwargs["flags"] |= SKIP
         else:
-            kwargs["flags"] = Flags(kwargs.get("flags")) & ~SKIP
+            kwargs["flags"] &= ~SKIP
             kwargs["start"] = None
             if parent:
                 with parent.lock:
@@ -844,7 +866,7 @@ class TestDefinition(object):
 
         found = len({tag for tag in only_tags if tag in tags}) > 0
         if not found:
-            kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
+            kwargs["flags"] |= SKIP
 
     def _apply_skip_tags(self, type, tags, kwargs):
         skip_tags = (kwargs.get("skip_tags", {}) or {}).get(type)
@@ -853,7 +875,7 @@ class TestDefinition(object):
 
         found = len({tag for tag in skip_tags if tag in tags}) > 0
         if found:
-            kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
+            kwargs["flags"] |= SKIP
 
     def _apply_only(self, name, kwargs):
         only = kwargs.get("only")
@@ -867,9 +889,9 @@ class TestDefinition(object):
                 break
 
         if not found:
-            kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
+            kwargs["flags"] |= SKIP
         else:
-            kwargs["flags"] = Flags(kwargs.get("flags")) & ~SKIP
+            kwargs["flags"] &= ~SKIP
 
     def _apply_skip(self, name, kwargs):
         skip = kwargs.get("skip")
@@ -878,7 +900,7 @@ class TestDefinition(object):
 
         for item in skip:
             if item.match(name, prefix=False):
-                kwargs["flags"] = Flags(kwargs.get("flags")) | SKIP
+                kwargs["flags"] |= SKIP
                 break
 
     def _apply_xflags(self, name, kwargs):
@@ -889,7 +911,7 @@ class TestDefinition(object):
         for pattern, item in xflags.items():
             if match(name, pattern):
                 set_flags, clear_flags = item
-                kwargs["flags"] = (Flags(kwargs.get("flags")) & ~Flags(clear_flags)) | Flags(set_flags)
+                kwargs["flags"] = (kwargs["flags"] & ~Flags(clear_flags)) | Flags(set_flags)
 
     def _parent_type_propagation(self, parent, kwargs):
         """Propagate parent test type if lower.
@@ -1007,13 +1029,16 @@ class TestDefinition(object):
             except (KeyboardInterrupt, Exception):
                 raise
 
+        if not self.parent:
+            sys.exit(0 if self.test.result else 1)
+
+        if isinstance(exception_value, KeyboardInterrupt):
+            raise KeyboardInterrupt
+
         # if test did not handle the exception in _exit then re-raise it
         if exception_value and not test__exit__:
             raise exception_value
 
-        if not self.parent:
-            sys.exit(0 if self.test.result else 1)
-
         if not self.test.result:
             if isinstance(self.test.result, Fail):
                 result = Fail(test=self.parent.name, message=self.test.result.message)
@@ -1227,6 +1252,7 @@ class TestDecorator(object):
                             _kwargs["name"] = str(example)
                             _kwargs["args"] = dict(kwargs.get("args", {}))
                             _kwargs["args"].update(vars(example))
+                            _kwargs.update(dict(examples.args))
                             _kwargs.update(dict(example._args))
                             _kwargs.pop("type", None)
                             _kwargs.pop("examples", None)
